@@ -84,7 +84,7 @@ class DataFreshness:
             self.run_number = self.previous_run_number + 1
             no_resources = self.no_resources_force_hash()
             if no_resources:
-                self.no_urls_to_check = int((no_resources / 30) + 1)
+                self.no_urls_to_check = no_resources
             else:
                 self.no_urls_to_check = default_no_urls_to_check
         else:
@@ -98,7 +98,7 @@ class DataFreshness:
         columns = [DBResource.id, DBDataset.updated_by_script]
         filters = [DBResource.dataset_id == DBDataset.id, DBResource.run_number == self.previous_run_number,
                    DBDataset.run_number == self.previous_run_number,
-                   DBResource.url.notlike('%{}%'.format(self.url_internal))]
+                   DBResource.url.notlike(f'%{self.url_internal}%')]
         query = self.session.query(*columns).filter(and_(*filters))
         noscriptupdate = 0
         noresources = 0
@@ -120,7 +120,72 @@ class DataFreshness:
         self.session.add(dbrun)
         self.session.commit()
 
-    def process_datasets(self, forced_hash_ids=None):
+    @staticmethod
+    def internal_what_updated(dbresource, url_substr):
+        what_updated = '%s-%s' % (url_substr, dbresource.what_updated)
+        dbresource.what_updated = what_updated
+
+    def process_resources(self, dataset_id, previous_dbdataset, resources, updated_by_script, hash_ids=None):
+        last_resource_updated = None
+        last_resource_modified = None
+        dataset_resources = list()
+        for resource in resources:
+            resource_id = resource['id']
+            dict_of_lists_add(self.resource_what_updated, 'total', resource_id)
+            url = resource['url']
+            name = resource['name']
+            revision_last_updated = parser.parse(resource['revision_last_updated'], ignoretz=True)
+            last_modified = parser.parse(resource['last_modified'], ignoretz=True)
+            if last_resource_modified:
+                if last_modified > last_resource_modified:
+                    last_resource_updated = resource_id
+                    last_resource_modified = last_modified
+            else:
+                last_resource_updated = resource_id
+                last_resource_modified = last_modified
+            dbresource = DBResource(run_number=self.run_number, id=resource_id, name=name,
+                                    dataset_id=dataset_id, url=url, last_modified=last_modified,
+                                    revision_last_updated=revision_last_updated, latest_of_modifieds=last_modified,
+                                    what_updated='firstrun')
+            if previous_dbdataset is not None:
+                try:
+                    previous_dbresource = self.session.query(DBResource).filter_by(id=resource_id,
+                                                                                   run_number=
+                                                                                   previous_dbdataset.run_number).one()
+                    if last_modified > previous_dbresource.last_modified:
+                        dbresource.what_updated = 'filestore'
+                    else:
+                        dbresource.last_modified = previous_dbresource.last_modified
+                        dbresource.what_updated = 'nothing'
+                    if last_modified <= previous_dbresource.latest_of_modifieds:
+                        dbresource.latest_of_modifieds = previous_dbresource.latest_of_modifieds
+                    dbresource.http_last_modified = previous_dbresource.http_last_modified
+                    dbresource.md5_hash = previous_dbresource.md5_hash
+                    dbresource.hash_last_modified = previous_dbresource.hash_last_modified
+                    dbresource.when_checked = previous_dbresource.when_checked
+
+                except NoResultFound:
+                    pass
+            self.session.add(dbresource)
+
+            if updated_by_script:
+                dict_of_lists_add(self.resource_what_updated, dbresource.what_updated, resource_id)
+                continue
+            if self.url_internal in url:
+                self.internal_what_updated(dbresource, 'internal')
+                dict_of_lists_add(self.resource_what_updated, dbresource.what_updated, resource_id)
+                continue
+            if hash_ids:
+                should_hash = resource_id in hash_ids
+            else:
+                should_hash = self.urls_to_check_count < self.no_urls_to_check \
+                              and (dbresource.when_checked is None or
+                                   self.now - dbresource.when_checked > datetime.timedelta(days=30))
+            resource_format = resource['format'].lower()
+            dataset_resources.append((url, resource_id, resource_format, dbresource.what_updated, should_hash))
+        return dataset_resources, last_resource_updated, last_resource_modified
+
+    def process_datasets(self, hash_ids=None):
         resources_to_check = list()
         datasets_to_check = dict()
         logger.info('Processing datasets')
@@ -163,11 +228,12 @@ class DataFreshness:
                                                                              id=dataset_id).one()
             except NoResultFound:
                 previous_dbdataset = None
-            dont_hash_script_update = False
+
             update_frequency = dataset.get('data_update_frequency')
-            updated_by_script = dataset.get('updated_by_script')
+            updated_by_script = None
             if update_frequency is not None:
                 update_frequency = int(update_frequency)
+                updated_by_script = dataset.get('updated_by_script')
                 if updated_by_script:
                     if 'freshness_ignore' in updated_by_script:
                         updated_by_script = None
@@ -178,12 +244,10 @@ class DataFreshness:
                         else:
                             try:
                                 updated_by_script = parser.parse(match.group(1), ignoretz=True)
-                                dont_hash_script_update = True
                             except ParserError:
                                 updated_by_script = None
             dataset_resources, last_resource_updated, last_resource_modified = \
-                self.process_resources(dataset_id, previous_dbdataset, resources, dont_hash_script_update,
-                                       forced_hash_ids=forced_hash_ids)
+                self.process_resources(dataset_id, previous_dbdataset, resources, updated_by_script, hash_ids=hash_ids)
             dataset_date = dataset.get('dataset_date')
             metadata_modified = parser.parse(dataset['metadata_modified'], ignoretz=True)
             if 'last_modified' in dataset:
@@ -209,6 +273,7 @@ class DataFreshness:
                     latest_of_modifieds = last_modified
             if updated_by_script and updated_by_script > latest_of_modifieds:
                 latest_of_modifieds = updated_by_script
+
             fresh = None
             if update_frequency is not None and not error:
                 if update_frequency == 0:
@@ -222,6 +287,7 @@ class DataFreshness:
                     self.adhoc_update += 1
                 else:
                     fresh = self.calculate_aging(latest_of_modifieds, update_frequency)
+
             dbdataset = DBDataset(run_number=self.run_number, id=dataset_id,
                                   dataset_date=dataset_date, update_frequency=update_frequency,
                                   review_date=review_date, last_modified=last_modified,
@@ -258,98 +324,25 @@ class DataFreshness:
                     if update_frequency is not None and update_frequency > 0:
                         fresh = self.calculate_aging(previous_dbdataset.latest_of_modifieds, update_frequency)
                         dbdataset.fresh = fresh
-
             self.session.add(dbdataset)
 
             update_string = '%s, Updated %s' % (self.aging_statuses[fresh], dbdataset.what_updated)
-            if fresh == 0 or update_frequency is None:
-                hash_forced = False
-                for url, resource_id, force_hash, what_updated in dataset_resources:
-                    if force_hash:
-                        hash_forced = True
-                        resources_to_check.append((url, resource_id, True, what_updated))
-                    else:
+            anyresourcestohash = False
+            for url, resource_id, resource_format, what_updated, should_hash in dataset_resources:
+                if not should_hash:
+                    if (fresh == 0 and update_frequency != 1) or update_frequency is None:
                         dict_of_lists_add(self.resource_what_updated, what_updated, resource_id)
-                if hash_forced:
-                    datasets_to_check[dataset_id] = update_string
-                else:
-                    dict_of_lists_add(self.dataset_what_updated, update_string, dataset_id)
-
-            else:
+                        continue
+                resources_to_check.append((url, resource_id, resource_format, what_updated))
+                self.urls_to_check_count += 1
+                anyresourcestohash = True
+            if anyresourcestohash:
                 datasets_to_check[dataset_id] = update_string
-                resources_to_check += dataset_resources
+            else:
+                dict_of_lists_add(self.dataset_what_updated, update_string, dataset_id)
         self.session.commit()
         return datasets_to_check, resources_to_check
 
-    def process_resources(self, dataset_id, previous_dbdataset, resources, dont_hash_script_update,
-                          forced_hash_ids=None):
-        last_resource_updated = None
-        last_resource_modified = None
-        dataset_resources = list()
-        for resource in resources:
-            resource_id = resource['id']
-            dict_of_lists_add(self.resource_what_updated, 'total', resource_id)
-            url = resource['url']
-            name = resource['name']
-            revision_last_updated = parser.parse(resource['revision_last_updated'], ignoretz=True)
-            last_modified = parser.parse(resource['last_modified'], ignoretz=True)
-            if last_resource_modified:
-                if last_modified > last_resource_modified:
-                    last_resource_updated = resource_id
-                    last_resource_modified = last_modified
-            else:
-                last_resource_updated = resource_id
-                last_resource_modified = last_modified
-            dbresource = DBResource(run_number=self.run_number, id=resource_id, name=name,
-                                    dataset_id=dataset_id, url=url, last_modified=last_modified,
-                                    revision_last_updated=revision_last_updated, latest_of_modifieds=last_modified,
-                                    what_updated='firstrun')
-            if previous_dbdataset is not None:
-                try:
-                    previous_dbresource = self.session.query(DBResource).filter_by(id=resource_id,
-                                                                                   run_number=
-                                                                                   previous_dbdataset.run_number).one()
-                    if last_modified > previous_dbresource.last_modified:
-                        dbresource.what_updated = 'filestore'
-                    else:
-                        dbresource.last_modified = previous_dbresource.last_modified
-                        dbresource.what_updated = 'nothing'
-                    if last_modified <= previous_dbresource.latest_of_modifieds:
-                        dbresource.latest_of_modifieds = previous_dbresource.latest_of_modifieds
-                    dbresource.http_last_modified = previous_dbresource.http_last_modified
-                    dbresource.md5_hash = previous_dbresource.md5_hash
-                    dbresource.when_hashed = previous_dbresource.when_hashed
-                    dbresource.when_checked = previous_dbresource.when_checked
-
-                except NoResultFound:
-                    pass
-            self.session.add(dbresource)
-
-            if self.url_internal in url:
-                self.internal_what_updated(dbresource, 'internal')
-                dict_of_lists_add(self.resource_what_updated, dbresource.what_updated, dbresource.id)
-                continue
-            if dont_hash_script_update:
-                forced_hash = False
-            else:
-                if forced_hash_ids:
-                    forced_hash = resource_id in forced_hash_ids
-                else:
-                    forced_hash = self.urls_to_check_count < self.no_urls_to_check \
-                                  and (dbresource.when_checked is None or
-                                       self.now - dbresource.when_checked > datetime.timedelta(days=30))
-            if forced_hash:
-                dataset_resources.append((url, resource_id, True, dbresource.what_updated))
-                self.urls_to_check_count += 1
-            else:
-                dataset_resources.append((url, resource_id, False, dbresource.what_updated))
-        return dataset_resources, last_resource_updated, last_resource_modified
-
-    @staticmethod
-    def internal_what_updated(dbresource, url_substr):
-        what_updated = '%s-%s' % (url_substr, dbresource.what_updated)
-        dbresource.what_updated = what_updated
-
     def check_urls(self, resources_to_check, user_agent, results=None, hash_results=None):
         def get_domain(x):
             return urlparse(x[0]).netloc
@@ -361,12 +354,12 @@ class DataFreshness:
 
         hash_check = list()
         for resource_id in results:
-            url, err, http_last_modified, hash, force_hash = results[resource_id]
+            url, resource_format, err, http_last_modified, hash = results[resource_id]
             if hash:
                 dbresource = self.session.query(DBResource).filter_by(id=resource_id,
                                                                       run_number=self.run_number).one()
                 if dbresource.md5_hash != hash:  # File changed
-                    hash_check.append((url, resource_id, force_hash))
+                    hash_check.append((url, resource_id, resource_format))
 
         if hash_results is None:  # pragma: no cover
             hash_check = list_distribute_contents(hash_check, get_domain)
@@ -379,7 +372,7 @@ class DataFreshness:
     def process_results(self, results, hash_results, resourcecls=Resource):
         datasets_latest_of_modifieds = dict()
         for resource_id in sorted(results):
-            url, err, http_last_modified, hash, force_hash = results[resource_id]
+            url, _, err, http_last_modified, hash = results[resource_id]
             dbresource = self.session.query(DBResource).filter_by(id=resource_id,
                                                                   run_number=self.run_number).one()
             dataset_id = dbresource.dataset_id
@@ -391,20 +384,20 @@ class DataFreshness:
                     dbresource.http_last_modified = http_last_modified
             if hash:
                 dbresource.when_checked = self.now
-                dbresource.when_hashed = self.now
                 if dbresource.md5_hash == hash:  # File unchanged
                     what_updated = self.add_what_updated(what_updated, 'same hash')
                 else:  # File updated
                     hash_to_set = hash
-                    hash_url, hash_err, hash_http_last_modified, hash_hash, force_hash = hash_results[resource_id]
+                    hash_url, _, hash_err, hash_http_last_modified, hash_hash = hash_results[resource_id]
                     if hash_http_last_modified:
                         if dbresource.http_last_modified is None or hash_http_last_modified > dbresource.http_last_modified:
                             dbresource.http_last_modified = hash_http_last_modified
                     if hash_hash:
                         if hash_hash == hash:
                             if dbresource.md5_hash is None:  # First occurrence of resource eg. first run - don't use hash
-                                                    # for last modified field (and hence freshness calculation)
-                                dbresource.what_updated = self.add_what_updated(what_updated, 'hash')
+                                # for last modified field (and hence freshness calculation)
+                                dbresource.what_updated = self.add_what_updated(what_updated,
+                                                                                'first hash')
                                 what_updated = dbresource.what_updated
                             else:
                                 # Check if hash has occurred before
@@ -415,6 +408,7 @@ class DataFreshness:
                                     what_updated = dbresource.what_updated
                                 else:
                                     what_updated, _ = self.set_latest_of_modifieds(dbresource, self.now, 'hash')
+                                    dbresource.hash_last_modified = self.now
                                     update_last_modified = True
                             dbresource.api = False
                         else:
